#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time
import datetime
import threading
import numpy as np

from LoggerViewDataTypes  import Rect
from LoggerViewDB         import Database

# ###########################################################
#
class Data():
  def __init__(self, filename):
    #print "vor DB-connect          ", datetime.datetime.now()
    self.filename=filename
    self.db=Database(filename)
    self.datacols=self.db.getColCount()               # Integer
    self.datarows=self.db.getRowCount()               # Integer
    self.coldesc=self.db.getColDescription()          # Liste
    self.minMaxTimestamp=self.db.getMinMaxTimestamp() # Tupel

    self.colorder=dict()  # definiert den Index der Spalten in self.arr
    for i in range(len(self.coldesc)):
      self.colorder.update({self.coldesc[i][0]:i+1})
    # Beispiel: {'2T': 3, '1H': 2, '2H': 4, '1T': 1}

    self.cache=dict()

    rows=1+(self.minMaxTimestamp[1]-self.minMaxTimestamp[0])/60
    self.baseIdx=self.minMaxTimestamp[0]
    #print "vor numpy create-array  ", datetime.datetime.now()
    self.arr=np.empty(shape=(rows, self.datacols+1), dtype=np.int64)
    #print "nach numpy, vor load    ", datetime.datetime.now()
    self.load()
    #print "7 tage sind einsortiert ", datetime.datetime.now()

    worker1=threading.Thread(target=self.loadStage2, name="load DB asynchronous")
    worker1.setDaemon(True)
    worker1.start()


  # ######################################################################
  # Lädt die Daten ins numpy-Array.
  def load(self):
    minTimestamp=datetime.datetime.fromtimestamp(self.minMaxTimestamp[0])
    maxTimestamp=datetime.datetime.fromtimestamp(self.minMaxTimestamp[1])
    self.min_new=maxTimestamp-datetime.timedelta(days=7)

    self.rowids=self.db.getRowIdentifiers((int(self.min_new.strftime("%s")), self.minMaxTimestamp[1]))
    self.datadict=self.db.getData((int(self.min_new.strftime("%s")), self.minMaxTimestamp[1]))
    #print "7 tage sind geladen     ", datetime.datetime.now()
    for rid in self.rowids:
      idx=(rid-self.baseIdx)/60
      self.arr[idx][0]=rid           # timestamp ablegen
      c=self.datadict[rid]           # Datenspalten holen
      for i in c:
        self.arr[idx][self.colorder[i[0]]]=i[1]   # Daten ablegen


  # ######################################################################
  # Lädt die restlichen Daten asynchron nach.
  def loadStage2(self):
    self.dbt=Database(self.filename)
    rowids=self.dbt.getRowIdentifiers((self.minMaxTimestamp[0], int(self.min_new.strftime("%s"))))
    datadict=self.dbt.getData((self.minMaxTimestamp[0], int(self.min_new.strftime("%s"))))
    #print "alles ist geladen       ", datetime.datetime.now()
    for rid in rowids:
      idx=(rid-self.baseIdx)/60
      self.arr[idx][0]=rid           # timestamp ablegen
      c=datadict[rid]                # Datenspalten holen
      for i in c:
        self.arr[idx][self.colorder[i[0]]]=i[1]   # Daten ablegen
    self.rowids.update(rowids)
    self.datadict.update(datadict)
    #print "alles ist einsortiert   ", datetime.datetime.now()


  # ######################################################################
  # Liefert ein Dictionary mit der Skala als key ("T" oder "H") und dem
  # Faktor, der darauf angewandt wurde, die den Messwert ganzzahlig zu
  # machen.
  def getFactors(self):
    return(self.db.getFactors())


  # ######################################################################
  # siehe DB-Klasse
  def getDataBoundaries(self):
    return(self.db.getDataBoundaries())
    # alte Version ... hat erst Daten liefern können, wenn das numpy-Array
    # komplett geladen war.
    dmin=self.arr.min(0)  # z.B.: [1483622580    199      0    -91      0]
    dmax=self.arr.max(0)  # z.B.: [1484287140    294    794     49    984]
    rd=dict()
    for n, i in self.colorder.items():
      rd.update({n:Rect(dmin[0], dmin[i], dmax[0], dmax[i])})
    return(rd)


  # ######################################################################
  # Liefert die Minimal- und Maximal-Werte aller Kurven als Dictionary,
  # aufgeteilt nach Seite der Skala.
  # Der key ist der Name der Kurve, value ist ein Rect().
  # Beispiel:
  #   { 'H': (x1=1483622580, y1=0,   x2=1484805540, y2=984, width=1182960, height=984),
  #     'T': (x1=1483622580, y1=-91, x2=1484805540, y2=294, width=1182960, height=385)  }
  def getFullDataBoundaries(self):
    b=self.getDataBoundaries()
    l_min=list()
    l_max=list()
    r_min=list()
    r_max=list()
    for k, v in b.items():
      if k.endswith("T"):   # linke Seite
        l_min.append(v.y1)
        l_max.append(v.y2)
        x_min=v.x1
        x_max=v.x2
      else:
        r_min.append(v.y1)
        r_max.append(v.y2)
        x_min=v.x1
        x_max=v.x2
    return({"T":Rect(x_min, min(l_min), x_max, max(l_max)),
            "H":Rect(x_min, min(r_min), x_max, max(r_max))})


  # ######################################################################
  # Wie __getLineList(), nur mit einem Cache für den letzten Satz
  # Kurven drüber.
  def getLineList(self, colnam, pix_rect, val_rect, win_h):
    #return(self.__getLineList(colnam, pix_rect, val_rect, win_h))
    if colnam in self.cache:
      opix_rect, oval_rect, owin_h, retlist=self.cache[colnam]
      if opix_rect==pix_rect and oval_rect==val_rect and owin_h==win_h:
        #print "from cache"
        return(retlist)
    retlist=self.__getLineList(colnam, pix_rect, val_rect, win_h)
    self.cache.update({colnam:(pix_rect, val_rect, win_h, retlist)})
    return(retlist)


  # ######################################################################
  # Liefert eine Liste von Listen für wx.dc.DrawLine()
  def __getLineList(self, colnam, pix_rect, val_rect, win_h):
    colidx=self.colorder[colnam]

    vw=float(val_rect.x2-val_rect.x1)/60  # Breite der Werte
    vh=val_rect.y2-val_rect.y1            # Höhe der Werte
    xs=float(pix_rect.width)/vw           # Skalierung für X
    ys=float(pix_rect.height)/vh          # Skalierung für Y

    retlist=list()
    linelist=list()
    xp=0
    if vw/pix_rect.width<=1: # wenn weniger als ein timestamp-Wert auf jedem Pixel liegt
      #print "direkt"
      for xl in xrange(val_rect.x1, val_rect.x2+60, 60):
        if xl in self.rowids:
          # (Messwert[timestamp] - SkalaUnterkante) * SkalierungsFaktor
          yd=(self.arr[(xl-self.baseIdx)/60][colidx]-val_rect.y1)*ys
          linelist.append((pix_rect.x1+int(round(xp*xs)), win_h-int(round(pix_rect.y1+yd))))
        else:                         # wenn eine Lücke in den Daten ist...
          if len(linelist)>0:
            retlist.append(linelist)  # ...neue Liste anfangen
            linelist=list()
        xp+=1
      retlist.append(linelist)
    else:
      #print "unique ts"
      last_x=None
      # erstmal Listen bilden, in denen jeder timestamp-Wert nur einmal
      # vorkommt und den Mittelwert der entspr. Daten enthält
      auts=list()
      uts=list()
      for xl in xrange(val_rect.x1, val_rect.x2, 60):
        if xl in self.rowids:
          x=pix_rect.x1+int(round(xp*xs))
          if x==last_x:
            y+=self.arr[(xl-self.baseIdx)/60][colidx]
            y/=2
          else:
            y=self.arr[(xl-self.baseIdx)/60][colidx]
            if last_x is not None:
              uts.append((last_x, y))
            last_x=x
        else:
          if len(uts)>0:
            auts.append(uts)
            uts=list()
            last_x=None
        xp+=1
      if last_x is not None:
        uts.append((last_x, y))
      auts.append(uts)
      xp=0
      for uts in auts:
        for x, y in uts:
          yd=(y-val_rect.y1)*ys
          linelist.append((x, win_h-int(round(pix_rect.y1+yd))))
          xp+=1
        retlist.append(linelist)
        linelist=list()
    return(retlist)
